#!/usr/bin/env python
# -*- coding: utf-8 -*-

# author: Pocholo
# date: 3/14/21

# Comments directed to http://gimpchat.com

# Installation:
# This script should be placed in the user plugins folder

# Copyright (C)

#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by   
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY# without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <https://www.gnu.org/licenses/>. 


from gimpfu import *
import math
import random
import gettext
import os, sys,time 

gettext.install("gimp20", gimp.locale_directory, unicode=True)

def pm_create_birthday_card(filename, customtext, fontname, textcolor, size):
          
    #Initiates the temporary state
    pdb.gimp_context_set_defaults()
    pdb.gimp_context_set_default_colors()
	
	#Create the background
    img = pdb.gimp_image_new(2000, 1300, RGB_IMAGE)   
    backLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Background", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_add_layer(img, backLayer, 0)
    pdb.gimp_layer_add_alpha(backLayer)
    pdb.gimp_drawable_fill(backLayer, FILL_WHITE)
    pdb.gimp_context_set_foreground((152, 44, 50))
    pdb.gimp_context_set_background((68, 140, 136))
    pdb.gimp_edit_blend(backLayer, 
                                0,  #blend mode - fg to bg rgb
                LAYER_MODE_NORMAL,  #paint mode 
                                0,  #gradient type - linear
                              100,  #opacity 
                                0,  #offset
                                0,  #repeat - none
                            FALSE,  #reverse
                             TRUE,  #supe-samle
                                3,  #max depth
                             0.20,  #threshold                            
                             TRUE,  #dither
                                0,  #x1
                              650,  #y1
                             2000,  #x2
                              650)  #y2  	
	
   	#Create bokeh
    dotLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Random Dots" , 100, LAYER_MODE_OVERLAY)
    pdb.gimp_image_add_layer(img, dotLayer, 0)
    pdb.gimp_layer_add_alpha(dotLayer)    
    pdb.gimp_drawable_fill(dotLayer, FILL_TRANSPARENT)
    ledge = min(img.width, img.height)
    outerradius = (10/100.0 * ledge)/2.0
    innerradius = (5/100.0 * ledge)/2.0
    maxradius = max(outerradius, innerradius)
    minradius = min(outerradius, innerradius)
    bunch = float(maxradius-minradius)/20;
    for i in range(0,int(20)):
        radius = int(random.randrange(int(bunch)) + (30-i-1)*(bunch)+ minradius)
        radius = int(random.randrange(int(bunch)) + (30-i-1)*(bunch)+ minradius)
        innerradius = int(5/100.0 * radius)
        cx = random.randrange(int(radius), int(img.width-radius))
        cy = random.randrange(int(radius), int(img.height-radius))
        pdb.gimp_image_select_ellipse(img, CHANNEL_OP_REPLACE, cx-radius, cy-radius, radius*2, radius*2)
        pdb.gimp_context_set_foreground("White")	
        pdb.gimp_edit_fill(dotLayer, FILL_FOREGROUND)
        pdb.gimp_image_select_ellipse(img, CHANNEL_OP_REPLACE, cx-innerradius, cy-innerradius, innerradius*2, innerradius*2)
        pdb.gimp_context_set_foreground((240, 240, 240))	
        pdb.gimp_edit_fill(dotLayer,FILL_FOREGROUND)
        pdb.gimp_drawable_offset(dotLayer, TRUE, OFFSET_BACKGROUND, img.width/2.0, img.height/2.0) #offset it so we get random all over.
        pdb.gimp_selection_none(img)
    pdb.plug_in_gauss(img, dotLayer, 30, 30, 1)
    
    #More bokeh
    dotLayer2 = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Random Dots #2" , 100, LAYER_MODE_OVERLAY)
    pdb.gimp_image_add_layer(img, dotLayer2, 0)
    pdb.gimp_layer_add_alpha(dotLayer2)    
    pdb.gimp_drawable_fill(dotLayer2, FILL_TRANSPARENT)
    ledge = min(img.width, img.height)
    outerradius = (12/100.0 * ledge)/2.0
    innerradius = (5/100.0 * ledge)/2.0
    maxradius = max(outerradius, innerradius)
    minradius = min(outerradius, innerradius)
    bunch = float(maxradius-minradius)/20;
    for i in range(0,int(20)):
        radius = int(random.randrange(int(bunch)) + (30-i-1)*(bunch)+ minradius)
        radius = int(random.randrange(int(bunch)) + (30-i-1)*(bunch)+ minradius)
        innerradius = int(5/100.0 * radius)
        cx = random.randrange(int(radius), int(img.width-radius))
        cy = random.randrange(int(radius), int(img.height-radius))
        pdb.gimp_image_select_ellipse(img, CHANNEL_OP_REPLACE, cx-radius, cy-radius, radius*2, radius*2)
        pdb.gimp_context_set_foreground("White")	
        pdb.gimp_edit_fill(dotLayer2, FILL_FOREGROUND)
        pdb.gimp_image_select_ellipse(img, CHANNEL_OP_REPLACE, cx-innerradius, cy-innerradius, innerradius*2, innerradius*2)
        pdb.gimp_context_set_foreground((240, 240, 240))	
        pdb.gimp_edit_fill(dotLayer2,FILL_FOREGROUND)
        pdb.gimp_drawable_offset(dotLayer2, TRUE, OFFSET_BACKGROUND, img.width/2.0, img.height/2.0) #offset it so we get random all over.
        pdb.gimp_selection_none(img)
    pdb.plug_in_gauss(img, dotLayer2, 70, 70, 1)
    layer = pdb.gimp_image_flatten(img)
    backLayer = pdb.gimp_image_get_layer_by_name(img, "Background")
    pdb.gimp_display_new(img)
    
    #Stamp frame
    frameLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Stamp frame" , 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_add_layer(img, frameLayer, 0)
    pdb.gimp_layer_add_alpha(frameLayer)    
    pdb.gimp_drawable_fill(frameLayer, FILL_TRANSPARENT)
    pdb.gimp_image_select_rectangle(img, 2, 200, 150, 1000, 1000)
    pdb.gimp_context_set_foreground((255, 255, 255))
    pdb.gimp_edit_bucket_fill(frameLayer, BUCKET_FILL_FG, LAYER_MODE_NORMAL, 100, 255, FALSE, 0, 0)
    pdb.plug_in_sel2path(img, frameLayer)
    pdb.gimp_brushes_set_brush("2. Hardness 100")
    pdb.gimp_context_set_brush_size(120)
    pdb.gimp_context_set_brush_spacing(0.90)
    pdb.gimp_context_set_brush_default_hardness()
    pdb.gimp_context_set_paint_method("gimp-eraser")
    pdb.gimp_context_set_stroke_method(STROKE_PAINT_METHOD)
    vectors1 = pdb.gimp_image_get_vectors_by_name(img, _("Selection"))
    pdb.gimp_drawable_edit_stroke_item(frameLayer, vectors1)   
    pdb.gimp_image_select_rectangle(img, 2, 239, 190, 920, 920)
    pdb.gimp_selection_invert(img)
    pdb.gimp_edit_clear(frameLayer)
    pdb.gimp_selection_none(img)
    photoLayer = pdb.gimp_item_transform_rotate(frameLayer, 6.1, TRUE, 0, 0)
    pdb.gimp_layer_translate(frameLayer, 0, -75)
    
    #Open a photo 820 x 820 px
    imagLayer = pdb.gimp_file_load_layer(img, filename)
    pdb.gimp_image_add_layer(img, imagLayer, 0)
    pdb.gimp_layer_add_alpha(imagLayer)
    pdb.gimp_layer_resize_to_image_size(imagLayer)   
    pdb.gimp_drawable_offset(imagLayer, TRUE, 0, 590, 240)
    imagLayer = pdb.gimp_item_transform_rotate(imagLayer, 6.1, TRUE, 0, 0)
    pdb.gimp_layer_resize_to_image_size(imagLayer) 
    pdb.gimp_drawable_offset(imagLayer, TRUE, 0, -297, -18)
    pdb.gimp_item_set_name(imagLayer, "Photo")
        
	#Paint the balloons
    balloonLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Balloons" , 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_add_layer(img, balloonLayer, 0)
    pdb.gimp_layer_add_alpha(balloonLayer)    
    pdb.gimp_drawable_fill(balloonLayer, FILL_TRANSPARENT)
    pdb.gimp_context_set_brush("Balloon #2")
    pdb.gimp_context_set_brush_size(800)
    pdb.gimp_context_set_brush_aspect_ratio(0)
    pdb.gimp_context_set_brush_angle(-14)
    pdb.gimp_context_set_brush_spacing(0.1)
    pdb.gimp_context_set_dynamics("Dynamics Off")  
    pdb.gimp_paintbrush_default(balloonLayer, 2, [1160, 455])
    pdb.gimp_context_set_brush_angle(14)
    pdb.gimp_context_set_brush("Balloon #5")
    pdb.gimp_paintbrush_default(balloonLayer, 2, [1265, 595])
    pdb.gimp_context_set_brush_angle(-14)
    pdb.gimp_context_set_brush("Balloon #4")
    pdb.gimp_paintbrush_default(balloonLayer, 2, [1165, 670])
    pdb.gimp_context_set_brush_angle(14)
    pdb.gimp_context_set_brush("Balloon #3")
    pdb.gimp_paintbrush_default(balloonLayer, 2, [218, 570])
    pdb.gimp_context_set_brush_angle(18)
    pdb.gimp_context_set_brush("Balloon #1")
    pdb.gimp_paintbrush_default(balloonLayer, 2, [180, 645])
    pdb.gimp_context_set_brush_angle(24)
    pdb.gimp_context_set_brush("Balloon #6")
    pdb.gimp_paintbrush_default(balloonLayer, 2, [250, 780])
	
	#Create the text		
    textW = size/0.5
    textH = size/0.5
    pdb.gimp_context_set_foreground(textcolor)
    textLayer = pdb.gimp_text_fontname(img, None, 0, 0, customtext, 10, 1, size, PIXELS, fontname)
    pdb.gimp_text_layer_set_justification(textLayer, 2)
    pdb.gimp_text_layer_set_letter_spacing(textLayer, 0)
    pdb.gimp_text_layer_set_line_spacing(textLayer, 0)
        
    #Resize text
    pdb.gimp_layer_scale(textLayer, textW, textH, TRUE)
                
    #Making the text centered	
    textLayer = pdb.gimp_image_get_active_layer(img)
    pdb.gimp_image_set_active_layer(img, textLayer)
    wh = pdb.gimp_drawable_width(textLayer)
    hh = pdb.gimp_drawable_height(textLayer)
    w = pdb.gimp_image_width(img)
    h = pdb.gimp_image_height(img)
    pdb.gimp_image_set_active_layer(img,textLayer)
    centerx=w/1.0
    centery=h/1.5
    adw=wh/1
    adh=hh/2
    sw=centerx - adw
    sh=centery - adh		
    pdb.gimp_layer_set_offsets(textLayer, sw, sh)
    pdb.gimp_layer_resize_to_image_size(textLayer)
    
    #Rename the text layer and move up
    pdb.gimp_item_set_name(textLayer, "Text")
    
    #Drop Shadow on the text
    pdb.python_layerfx_drop_shadow(img, textLayer,
										  ("Black"),   #color
											    100,   #opacity
												  0,   #contou - linear
												  0,   #noise
												  3,   #mode - multiply
												  7,   #spread
												 10,   #size
											    120,   #offset_angle
											      7,   #offset_distance
											   TRUE,   #knockout
											  FALSE)   #merge
	
	#Bevel and emboss
    pdb.python_layerfx_bevel_emboss(img, textLayer, 
									                1,   #style inner bevel
								                   65,   #depth 
								                    0,   #direction 
								                   10,   #size 
									                5,   #soften
								                  120,   #angle 
								                   30,   #altitude 
									                3,   #gloss_contour - cove deep
							                  "White",   #highlight_color 
								                    5,   #highlight_mode screen
								                  100,   #highlight_opacity
							                  "Black",   #shadow_color 
								                    3,   #shadow_mode multiply
								                  100,   #shadow_opacity 
								                    5,   #surface_contour - gaussian
								                FALSE,   #use_texture 
							              "Dried mud",   #pattern 
								                  100,   #scale 
								                  100,   #tex_depth 
								                 TRUE,   #invert 
								                FALSE)   #merge	
	
    pdb.gimp_drawable_hue_saturation(textLayer, 0, 0, 45, 0, 0)
		
    backLayer = pdb.gimp_image_get_layer_by_name(img, "Background")
    pdb.gimp_image_set_active_layer(img, backLayer)
         	
	#Set gimp to default
    pdb.gimp_context_set_defaults()
    pdb.gimp_context_set_default_colors()
   	
register(
    "pm_create_birthday_card",
    "Creates a Birthday card",
    "Creates a Birthday card",
    "Pocholo",
    "Pocholo",
    "2021",
    "Create a Birthday Card",
    "",
    [
    (PF_FILENAME, "filename", "Photo 820 x 820 px:", None),
    (PF_TEXT, "customtext", "Text string", 'Happy\nBirthday'),
    (PF_FONT, "fontname", "Font name", "Femina"),
    (PF_COLOR, "textcolor", "Text color:", (255, 255, 255)),
    (PF_SPINNER, "size", "Text size", 420, (300, 420, 1)), 
    ],
    [],
    pm_create_birthday_card, menu="<Image>/Pocholo-scripts/Create a Birthday Card",
    domain=("gimp20", gimp.locale_directory))
main()						   		